import asyncio
import os
import websockets
import random
import string
from datetime import datetime

connected_clients = {}  # 接続されているクライアントを保持する辞書

def generate_id():
    # クライアントIDをランダムに生成する関数
    return ''.join(random.choices(string.ascii_letters + string.digits, k=8))

async def handler(websocket, path):
    # 各クライアント接続のための非同期ハンドラー関数
    client_id = generate_id()
    while client_id in connected_clients:
        client_id = generate_id()

    connected_clients[client_id] = websocket  # クライアントIDとWebSocket接続をマッピング

    # test.txt の絶対パスを設定
    file_path = "~~~/WebSocket/websocket/server/public_html/cgi-bin/test.txt"
    
    # 新しいクライアントに過去のメッセージを送信
    if os.path.exists(file_path):
        with open(file_path, "r") as f:
            past_messages = f.read()
            await websocket.send(past_messages)

    try:
        async for message in websocket:
            # クライアントからメッセージを受信
            timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            formatted_message = f"名無しさん＠{client_id} {timestamp}\n{message}\n\n"
            
            # 受信したメッセージをファイルに保存
            with open("test.txt", "a") as f:
                f.write(formatted_message)
            
            # 接続されている全クライアントにメッセージを送信
            for client in connected_clients.values():
                await client.send(formatted_message)

    except:
        # 例外発生時の処理（ここでは何もしない）
        pass
    finally:
        # WebSocket接続が閉じられると、クライアントリストから削除
        del connected_clients[client_id]

# WebSocketサーバーを起動
start_server = websockets.serve(handler, "0.0.0.0", 8084)

# イベントループを開始し、サーバーを実行
asyncio.get_event_loop().run_until_complete(start_server)
asyncio.get_event_loop().run_forever()